@@ -365,11 +365,17 @@ static int uclist_add_signature(const unsigned int id,
 
 	/* look for duplicates, stop on the first */
 	while (likely(*pnext) && (*pnext)->cpuid == cpuid) {
-		if ((*pnext)->pfm == pf_mask && (*pnext)->uc_rev == uc_rev) {
-			res = EEXIST;
-			flags |= INTEL_UCLE_DUPSIG;
-			d = *pnext;
-			break;
+		if ((*pnext)->uc_rev == uc_rev) {
+			if ((*pnext)->pfm == pf_mask) {
+				res = EEXIST;
+				flags |= INTEL_UCLE_DUPSIG;
+				d = *pnext;
+				break;
+			} else if (((*pnext)->pfm & pf_mask)) {
+				/* not a DUPSIG, but we need to
+				 * compare the payloads */
+				d = *pnext;
+			}
 		}
 		pnext = &((*pnext)->next);
 	}
@@ -1005,7 +1011,7 @@ static int load_intel_microcode_bin(int fd,
 
 	/* sanity check size */
 	if (mcb_size < INTEL_UC_MINSIZE) {
-		err = (mcb_size) ? EINVAL : ENOENT;
+		err = (mcb_size && !scan) ? EINVAL : ENOENT;
 		goto err_out;
 	}
 
@@ -1057,7 +1063,7 @@ static int load_intel_microcode_bin(int fd,
 
 	mcb_size = pos;
 	if (mcb_size < INTEL_UC_MINSIZE) {
-		err = (mcb_size) ? EINVAL : ENOENT;
+		err = (mcb_size && !scan) ? EINVAL : ENOENT;
 		goto err_out;
 	}
 
@@ -2924,7 +2930,7 @@ static const struct argp_option cmdline_options[] = {
 	   "Abort on broken microcode entries (default)",
 	  56 },
 
-	{ 0 },
+	{ .name = NULL },
 };
 static const char cmdline_nonarg_doc[] = "[[-t<type>] filename] ...";
 
